# -*- coding: utf-8 -*-
"""Cisco DNA Center Health and Performance API wrapper.

Copyright (c) 2024 Cisco Systems.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""



from builtins import *



from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class HealthAndPerformance(object):
    """Cisco DNA Center Health and Performance API (version: 2.3.7.6).

    Wraps the DNA Center Health and Performance
    API and exposes the API as native Python
    methods that return native Python objects.

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new HealthAndPerformance
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the DNA Center service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(HealthAndPerformance, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def retrieves_all_the_validation_sets_v1(self,
                                             view=None,
                                             headers=None,
                                             **request_parameters):
        """Retrieves all the validation sets and optionally the contained validations .

        Args:
            view(str): view query parameter. When the query parameter `view=DETAIL` is passed, all validation
                sets and associated validations will be returned. When the query parameter
                `view=DEFAULT` is passed, only validation sets metadata will be returned. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!retrieves-all-the-validation-sets
        """
        check_type(headers, dict)
        check_type(view, str)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'view':
                view,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/diagnosticValidationSets')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_d6fc1397d48d52449923716aff009d3c_v2_3_7_6', json_data)

    def retrieves_validation_details_for_a_validation_set_v1(self,
                                                             id,
                                                             headers=None,
                                                             **request_parameters):
        """Retrieves validation details for the given validation set id .

        Args:
            id(str): id path parameter. Validation set id .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!retrieves-validation-details-for-a-validation-set
        """
        check_type(headers, dict)
        check_type(id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/diagnosticValidationSets/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_d95307fdbf5b169d9d05e3151f61ac_v2_3_7_6', json_data)

    def retrieves_the_list_of_validation_workflows_v1(self,
                                                      end_time=None,
                                                      limit=None,
                                                      offset=None,
                                                      run_status=None,
                                                      start_time=None,
                                                      headers=None,
                                                      **request_parameters):
        """Retrieves the workflows that have been successfully submitted and are currently available. This is sorted by
        `submitTime` .

        Args:
            start_time(int): startTime query parameter. Workflows started after the given time (as milliseconds
                since UNIX epoch). .
            end_time(int): endTime query parameter. Workflows started before the given time (as milliseconds since
                UNIX epoch). .
            run_status(str): runStatus query parameter. Execution status of the workflow. If the workflow is
                successfully submitted, runStatus is `PENDING`. If the workflow execution has started,
                runStatus is `IN_PROGRESS`. If the workflow executed is completed with all validations
                executed, runStatus is `COMPLETED`. If the workflow execution fails while running
                validations, runStatus is `FAILED`. .
            offset(int): offset query parameter. The first record to show for this page; the first record is
                numbered 1. .
            limit(int): limit query parameter. The number of records to show for this page. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!retrieves-the-list-of-validation-workflows
        """
        check_type(headers, dict)
        check_type(start_time, int)
        check_type(end_time, int)
        check_type(run_status, str)
        check_type(offset, int)
        check_type(limit, int)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'startTime':
                start_time,
            'endTime':
                end_time,
            'runStatus':
                run_status,
            'offset':
                offset,
            'limit':
                limit,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/diagnosticValidationWorkflows')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_a53d325f85e5549b7c5957c6ecbd891_v2_3_7_6', json_data)

    def submits_the_workflow_for_executing_validations_v1(self,
                                                          description=None,
                                                          name=None,
                                                          validationSetIds=None,
                                                          headers=None,
                                                          payload=None,
                                                          active_validation=True,
                                                          **request_parameters):
        """Submits the workflow for executing the validations for the given validation specifications .

        Args:
            description(string): Health and Performance's Description of the workflow to run .
            name(string): Health and Performance's Name of the workflow to run. It must be unique. .
            validationSetIds(list): Health and Performance's List of validation set ids  (list of strings).
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!submits-the-workflow-for-executing-validations
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        _payload = {
            'name':
                name,
            'description':
                description,
            'validationSetIds':
                validationSetIds,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_cf9d39cef5e95bb9bd48d5f86e094c99_v2_3_7_6')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/diagnosticValidationWorkflows')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload)

        return self._object_factory('bpm_cf9d39cef5e95bb9bd48d5f86e094c99_v2_3_7_6', json_data)

    def retrieves_the_count_of_validation_workflows_v1(self,
                                                       end_time=None,
                                                       run_status=None,
                                                       start_time=None,
                                                       headers=None,
                                                       **request_parameters):
        """Retrieves the count of workflows that have been successfully submitted and are currently available.  .

        Args:
            start_time(int): startTime query parameter. Workflows started after the given time (as milliseconds
                since UNIX epoch). .
            end_time(int): endTime query parameter. Workflows started before the given time (as milliseconds since
                UNIX epoch). .
            run_status(str): runStatus query parameter. Execution status of the workflow. If the workflow is
                successfully submitted, runStatus is `PENDING`. If the workflow execution has started,
                runStatus is `IN_PROGRESS`. If the workflow executed is completed with all validations
                executed, runStatus is `COMPLETED`. If the workflow execution fails while running
                validations, runStatus is `FAILED`. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!retrieves-the-count-of-validation-workflows
        """
        check_type(headers, dict)
        check_type(start_time, int)
        check_type(end_time, int)
        check_type(run_status, str)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'startTime':
                start_time,
            'endTime':
                end_time,
            'runStatus':
                run_status,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/diagnosticValidationWorkflows/count')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_b174a2fc5171520d9423c9a50f7394e7_v2_3_7_6', json_data)

    def deletes_a_validation_workflow_v1(self,
                                         id,
                                         headers=None,
                                         **request_parameters):
        """Deletes the workflow for the given id .

        Args:
            id(str): id path parameter. Workflow id .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!deletes-a-validation-workflow
        """
        check_type(headers, dict)
        check_type(id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/diagnosticValidationWorkflows/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.delete(endpoint_full_url, params=_params,
                                             headers=_headers)
        else:
            json_data = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_b3ab76a74dae51fabf39b2ad85c3c58f_v2_3_7_6', json_data)

    def retrieves_validation_workflow_details_v1(self,
                                                 id,
                                                 headers=None,
                                                 **request_parameters):
        """Retrieves workflow details for a workflow id .

        Args:
            id(str): id path parameter. Workflow id .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!retrieves-validation-workflow-details
        """
        check_type(headers, dict)
        check_type(id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/diagnosticValidationWorkflows/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_c36c30b8c5ddfbf9ccf36db5dd68a_v2_3_7_6', json_data)

    def system_health_v1(self,
                      domain=None,
                      limit=None,
                      offset=None,
                      subdomain=None,
                      summary=None,
                      headers=None,
                      **request_parameters):
        """This API retrieves the latest system events  .

        Args:
            summary(bool): summary query parameter. Fetch the latest high severity event .
            domain(str): domain query parameter. Fetch system events with this domain. Possible values of
                domain are listed here : /dna/platform/app/consumer-portal/developer-toolkit/events .
            subdomain(str): subdomain query parameter. Fetch system events with this subdomain. Possible
                values of subdomain are listed here : /dna/platform/app/consumer-portal/developer-
                toolkit/events .
            limit(int): limit query parameter.
            offset(int): offset query parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!system-health-api
        """
        check_type(headers, dict)
        check_type(summary, bool)
        check_type(domain, str)
        check_type(subdomain, str)
        check_type(limit, int)
        check_type(offset, int)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'summary':
                summary,
            'domain':
                domain,
            'subdomain':
                subdomain,
            'limit':
                limit,
            'offset':
                offset,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/diagnostics/system/health')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_d0acccfae6885bc28f8f39c67f4acfc1_v2_3_7_6', json_data)

    def system_health_count_v1(self,
                            domain=None,
                            subdomain=None,
                            headers=None,
                            **request_parameters):
        """This API gives the count of the latest system events .

        Args:
            domain(str): domain query parameter. Fetch system events with this domain. Possible values of
                domain are listed here : /dna/platform/app/consumer-portal/developer-toolkit/events .
            subdomain(str): subdomain query parameter. Fetch system events with this subdomain. Possible
                values of subdomain are listed here : /dna/platform/app/consumer-portal/developer-
                toolkit/events .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!system-health-count-api
        """
        check_type(headers, dict)
        check_type(domain, str)
        check_type(subdomain, str)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'domain':
                domain,
            'subdomain':
                subdomain,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/diagnostics/system/health/count')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_f6dd603bc35db1948f31c782a37647_v2_3_7_6', json_data)

    def system_performance_v1(self,
                           end_time=None,
                           function=None,
                           kpi=None,
                           start_time=None,
                           headers=None,
                           **request_parameters):
        """Retrieves the aggregated metrics (total, average or maximum) of cluster key performance indicators (KPIs), such
        as CPU utilization, memory utilization or network rates recorded within a specified time period. The
        data will be available from the past 24 hours. .

        Args:
            kpi(str): kpi query parameter. Valid values: cpu,memory,network .
            function(str): function query parameter. Valid values: sum,average,max .
            start_time(int): startTime query parameter. This is the epoch start time in milliseconds from which
                performance indicator need to be fetched .
            end_time(int): endTime query parameter. This is the epoch end time in milliseconds upto which
                performance indicator need to be fetched .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!system-performance-api
        """
        check_type(headers, dict)
        check_type(kpi, str)
        check_type(function, str)
        check_type(start_time, int)
        check_type(end_time, int)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'kpi':
                kpi,
            'function':
                function,
            'startTime':
                start_time,
            'endTime':
                end_time,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/diagnostics/system/performance')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_cfcb7a875f215cb4ba59be38abb871e6_v2_3_7_6', json_data)

    def system_performance_historical_v1(self,
                                      end_time=None,
                                      kpi=None,
                                      start_time=None,
                                      headers=None,
                                      **request_parameters):
        """Retrieves the average values of cluster key performance indicators (KPIs), like CPU utilization, memory
        utilization or network rates grouped by time intervals within a specified time range. The data will be
        available from the past 24 hours. .

        Args:
            kpi(str): kpi query parameter. Fetch historical data for this kpi. Valid values:
                cpu,memory,network .
            start_time(int): startTime query parameter. This is the epoch start time in milliseconds from which
                performance indicator need to be fetched .
            end_time(int): endTime query parameter. This is the epoch end time in milliseconds upto which
                performance indicator need to be fetched .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!system-performance-historical-api
        """
        check_type(headers, dict)
        check_type(kpi, str)
        check_type(start_time, int)
        check_type(end_time, int)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'kpi':
                kpi,
            'startTime':
                start_time,
            'endTime':
                end_time,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/diagnostics/system/performance/histor'
                 + 'y')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_f131d712dc253dca528c0298b3e41c6_v2_3_7_6', json_data)

                
    
    # Alias Function
    def system_performance(self,
                           end_time=None,
                           function=None,
                           kpi=None,
                           start_time=None,
                           headers=None,
                           **request_parameters):
        """ This function is an alias of system_performance_v1 .
        Args:
            kpi(basestring): kpi query parameter. Valid values: cpu,memory,network .
            function(basestring): function query parameter. Valid values: sum,average,max .
            start_time(int): startTime query parameter. This is the epoch start time in milliseconds from which
                performance indicator need to be fetched .
            end_time(int): endTime query parameter. This is the epoch end time in milliseconds upto which
                performance indicator need to be fetched .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of system_performance_v1 .
        """ 
        return self.system_performance_v1(
                    end_time=end_time,
                    function=function,
                    kpi=kpi,
                    start_time=start_time,
                    headers=headers,
                    **request_parameters
        )
                
    
    # Alias Function
    def system_performance_historical(self,
                                      end_time=None,
                                      kpi=None,
                                      start_time=None,
                                      headers=None,
                                      **request_parameters):
        """ This function is an alias of system_performance_historical_v1 .
        Args:
            kpi(basestring): kpi query parameter. Fetch historical data for this kpi. Valid values:
                cpu,memory,network .
            start_time(int): startTime query parameter. This is the epoch start time in milliseconds from which
                performance indicator need to be fetched .
            end_time(int): endTime query parameter. This is the epoch end time in milliseconds upto which
                performance indicator need to be fetched .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of system_performance_historical_v1 .
        """ 
        return self.system_performance_historical_v1(
                    end_time=end_time,
                    kpi=kpi,
                    start_time=start_time,
                    headers=headers,
                    **request_parameters
        )
                
    
    # Alias Function
    def system_health(self,
                      domain=None,
                      limit=None,
                      offset=None,
                      subdomain=None,
                      summary=None,
                      headers=None,
                      **request_parameters):
        """ This function is an alias of system_health_v1 .
        Args:
            summary(bool): summary query parameter. Fetch the latest high severity event .
            domain(basestring): domain query parameter. Fetch system events with this domain. Possible values of
                domain are listed here : /dna/platform/app/consumer-portal/developer-toolkit/events .
            subdomain(basestring): subdomain query parameter. Fetch system events with this subdomain. Possible
                values of subdomain are listed here : /dna/platform/app/consumer-portal/developer-
                toolkit/events .
            limit(int): limit query parameter.
            offset(int): offset query parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of system_health_v1 .
        """ 
        return self.system_health_v1(
                    domain=domain,
                    limit=limit,
                    offset=offset,
                    subdomain=subdomain,
                    summary=summary,
                    headers=headers,
                    **request_parameters
        )
                
    
    # Alias Function
    def deletes_a_validation_workflow(self,
                                         id,
                                         headers=None,
                                         **request_parameters):
        """ This function is an alias of deletes_a_validation_workflow_v1 .
        Args:
            id(basestring): id path parameter. Workflow id .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of deletes_a_validation_workflow_v1 .
        """ 
        return self.deletes_a_validation_workflow_v1(
                    id=id,
                    headers=headers,
                    **request_parameters
        )
                
    
    # Alias Function
    def retrieves_validation_details_for_a_validation_set(self,
                                                             id,
                                                             headers=None,
                                                             **request_parameters):
        """ This function is an alias of retrieves_validation_details_for_a_validation_set_v1 .
        Args:
            id(basestring): id path parameter. Validation set id .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of retrieves_validation_details_for_a_validation_set_v1 .
        """ 
        return self.retrieves_validation_details_for_a_validation_set_v1(
                    id=id,
                    headers=headers,
                    **request_parameters
        )
                
    
    # Alias Function
    def retrieves_the_count_of_validation_workflows(self,
                                                       end_time=None,
                                                       run_status=None,
                                                       start_time=None,
                                                       headers=None,
                                                       **request_parameters):
        """ This function is an alias of retrieves_the_count_of_validation_workflows_v1 .
        Args:
            start_time(int): startTime query parameter. Workflows started after the given time (as milliseconds
                since UNIX epoch). .
            end_time(int): endTime query parameter. Workflows started before the given time (as milliseconds since
                UNIX epoch). .
            run_status(basestring): runStatus query parameter. Execution status of the workflow. If the workflow is
                successfully submitted, runStatus is `PENDING`. If the workflow execution has started,
                runStatus is `IN_PROGRESS`. If the workflow executed is completed with all validations
                executed, runStatus is `COMPLETED`. If the workflow execution fails while running
                validations, runStatus is `FAILED`. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of retrieves_the_count_of_validation_workflows_v1 .
        """ 
        return self.retrieves_the_count_of_validation_workflows_v1(
                    end_time=end_time,
                    run_status=run_status,
                    start_time=start_time,
                    headers=headers,
                    **request_parameters
        )
                
    
    # Alias Function
    def retrieves_the_list_of_validation_workflows(self,
                                                      end_time=None,
                                                      limit=None,
                                                      offset=None,
                                                      run_status=None,
                                                      start_time=None,
                                                      headers=None,
                                                      **request_parameters):
        """ This function is an alias of retrieves_the_list_of_validation_workflows_v1 .
        Args:
            start_time(int): startTime query parameter. Workflows started after the given time (as milliseconds
                since UNIX epoch). .
            end_time(int): endTime query parameter. Workflows started before the given time (as milliseconds since
                UNIX epoch). .
            run_status(basestring): runStatus query parameter. Execution status of the workflow. If the workflow is
                successfully submitted, runStatus is `PENDING`. If the workflow execution has started,
                runStatus is `IN_PROGRESS`. If the workflow executed is completed with all validations
                executed, runStatus is `COMPLETED`. If the workflow execution fails while running
                validations, runStatus is `FAILED`. .
            offset(int): offset query parameter. The first record to show for this page; the first record is
                numbered 1. .
            limit(int): limit query parameter. The number of records to show for this page. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of retrieves_the_list_of_validation_workflows_v1 .
        """ 
        return self.retrieves_the_list_of_validation_workflows_v1(
                    end_time=end_time,
                    limit=limit,
                    offset=offset,
                    run_status=run_status,
                    start_time=start_time,
                    headers=headers,
                    **request_parameters
        )
                
    
    # Alias Function
    def system_health_count(self,
                            domain=None,
                            subdomain=None,
                            headers=None,
                            **request_parameters):
        """ This function is an alias of system_health_count_v1 .
        Args:
            domain(basestring): domain query parameter. Fetch system events with this domain. Possible values of
                domain are listed here : /dna/platform/app/consumer-portal/developer-toolkit/events .
            subdomain(basestring): subdomain query parameter. Fetch system events with this subdomain. Possible
                values of subdomain are listed here : /dna/platform/app/consumer-portal/developer-
                toolkit/events .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of system_health_count_v1 .
        """ 
        return self.system_health_count_v1(
                    domain=domain,
                    subdomain=subdomain,
                    headers=headers,
                    **request_parameters
        )
                
    
    # Alias Function
    def retrieves_validation_workflow_details(self,
                                                 id,
                                                 headers=None,
                                                 **request_parameters):
        """ This function is an alias of retrieves_validation_workflow_details_v1 .
        Args:
            id(basestring): id path parameter. Workflow id .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of retrieves_validation_workflow_details_v1 .
        """ 
        return self.retrieves_validation_workflow_details_v1(
                    id=id,
                    headers=headers,
                    **request_parameters
        )
                
    
    # Alias Function
    def submits_the_workflow_for_executing_validations(self,
                                                          description=None,
                                                          name=None,
                                                          validationSetIds=None,
                                                          headers=None,
                                                          payload=None,
                                                          active_validation=True,
                                                          **request_parameters):
        """ This function is an alias of submits_the_workflow_for_executing_validations_v1 .
        Args:
            description(string): Health and Performance's Description of the workflow to run .
            name(string): Health and Performance's Name of the workflow to run. It must be unique. .
            validationSetIds(list): Health and Performance's List of validation set ids  (list of strings).
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of submits_the_workflow_for_executing_validations_v1 .
        """ 
        return self.submits_the_workflow_for_executing_validations_v1(
                    description=description,
                    name=name,
                    validationSetIds=validationSetIds,
                    headers=headers,
                    payload=payload,
                    active_validation=active_validation,
                    **request_parameters
        )
                
    
    # Alias Function
    def retrieves_all_the_validation_sets(self,
                                             view=None,
                                             headers=None,
                                             **request_parameters):
        """ This function is an alias of retrieves_all_the_validation_sets_v1 .
        Args:
            view(basestring): view query parameter. When the query parameter `view=DETAIL` is passed, all validation
                sets and associated validations will be returned. When the query parameter
                `view=DEFAULT` is passed, only validation sets metadata will be returned. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of retrieves_all_the_validation_sets_v1 .
        """ 
        return self.retrieves_all_the_validation_sets_v1(
                    view=view,
                    headers=headers,
                    **request_parameters
        )





# -*- coding: utf-8 -*-
"""Cisco DNA Center Sites API wrapper.

Copyright (c) 2019-2021 Cisco Systems.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from builtins import *


from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class Sites(object):
    """Cisco DNA Center Sites API (version: 2.2.2.3).

    Wraps the DNA Center Sites
    API and exposes the API as native Python
    methods that return native Python objects.

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new Sites
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the DNA Center service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(Sites, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def get_membership(self,
                       site_id,
                       device_family=None,
                       limit=None,
                       offset=None,
                       serial_number=None,
                       headers=None,
                       **request_parameters):
        """Getting the site children details and device details. .

        Args:
            site_id(str): siteId path parameter. Site id to retrieve device associated with the site. .
            offset(str,int): offset query parameter. offset/starting row .
            limit(str,int): limit query parameter. Number of sites to be retrieved .
            device_family(str): deviceFamily query parameter. Device family name  .
            serial_number(str): serialNumber query parameter. Device serial number .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(offset, (int, str))
        check_type(limit, (int, str))
        check_type(device_family, str)
        check_type(serial_number, str)
        check_type(site_id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'offset':
                offset,
            'limit':
                limit,
            'deviceFamily':
                device_family,
            'serialNumber':
                serial_number,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'siteId': site_id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/membership/{siteId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_ca11e0b5f8d91395e2462a9cfdc_v2_2_2_3', json_data)

    def create_site(self,
                    site=None,
                    type=None,
                    headers=None,
                    payload=None,
                    active_validation=True,
                    **request_parameters):
        """Creates site with area/building/floor with specified hierarchy. .

        Args:
            site(object): Sites's site.
            type(string): Sites's Type of site to create (eg: area, building, floor) . Available values are 'area',
                'building' and 'floor'.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if '__runsync' in headers:
                check_type(headers.get('__runsync'),
                           bool, may_be_none=False)
            if '__timeout' in headers:
                check_type(headers.get('__timeout'),
                           int)
            if '__persistbapioutput' in headers:
                check_type(headers.get('__persistbapioutput'),
                           bool, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        _payload = {
            'type':
                type,
            'site':
                site,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_bce8e6b307ce52dd8f5546fbd78e05ee_v2_2_2_3')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/site')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload)

        return self._object_factory('bpm_bce8e6b307ce52dd8f5546fbd78e05ee_v2_2_2_3', json_data)

    def get_site(self,
                 limit=None,
                 name=None,
                 offset=None,
                 site_id=None,
                 type=None,
                 headers=None,
                 **request_parameters):
        """Get site using siteNameHierarchy/siteId/type ,return all sites if these parameters are not given as input. .

        Args:
            name(str): name query parameter. siteNameHierarchy (ex: global/groupName) .
            site_id(str): siteId query parameter. Site id to which site details to retrieve. .
            type(str): type query parameter. type (ex: area, building, floor) .
            offset(str,int): offset query parameter. offset/starting row. The default value is 1 .
            limit(str,int): limit query parameter. Number of sites to be retrieved. The default value is 500 .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(name, str)
        check_type(site_id, str)
        check_type(type, str)
        check_type(offset, (int, str))
        check_type(limit, (int, str))
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'name':
                name,
            'siteId':
                site_id,
            'type':
                type,
            'offset':
                offset,
            'limit':
                limit,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/site')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_dbdd6074bedc59b9a3edd6477897d659_v2_2_2_3', json_data)

    def get_site_health(self,
                        timestamp=None,
                        headers=None,
                        **request_parameters):
        """Returns Overall Health information for all sites .

        Args:
            timestamp(str): timestamp query parameter. Epoch time(in milliseconds) when the Site Hierarchy
                data is required .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(timestamp, str)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'timestamp':
                timestamp,
        }

        if _params['timestamp'] is None:
            _params['timestamp'] = ''
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/site-health')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_ae4b592f66035f24b55028f79c1b7290_v2_2_2_3', json_data)

    def get_site_count(self,
                       site_id=None,
                       headers=None,
                       **request_parameters):
        """API to get site count  .

        Args:
            site_id(str): siteId query parameter. Site id to retrieve site count. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(site_id, str)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'siteId':
                site_id,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/site/count')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_e7a025fbe2c452fc82eedd5c50104aba_v2_2_2_3', json_data)

    def update_site(self,
                    site_id,
                    site=None,
                    type=None,
                    headers=None,
                    payload=None,
                    active_validation=True,
                    **request_parameters):
        """Update site area/building/floor with specified hierarchy and new values .

        Args:
            site(object): Sites's site.
            type(string): Sites's Type. Available values are 'area', 'building' and 'floor'.
            site_id(str): siteId path parameter. Site id to which site details to be updated. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(site_id, str,
                   may_be_none=False)
        if headers is not None:
            if '__runsync' in headers:
                check_type(headers.get('__runsync'),
                           bool)
            if '__timeout' in headers:
                check_type(headers.get('__timeout'),
                           int)
            if '__persistbapioutput' in headers:
                check_type(headers.get('__persistbapioutput'),
                           bool, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'siteId': site_id,
        }
        _payload = {
            'type':
                type,
            'site':
                site,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_df9908ad265e83ab77d73803925678_v2_2_2_3')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/site/{siteId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload,
                                          headers=_headers)
        else:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload)

        return self._object_factory('bpm_df9908ad265e83ab77d73803925678_v2_2_2_3', json_data)

    def delete_site(self,
                    site_id,
                    headers=None,
                    **request_parameters):
        """Delete site with area/building/floor by siteId. .

        Args:
            site_id(str): siteId path parameter. Site id to which site details to be deleted. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(site_id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'siteId': site_id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/site/{siteId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.delete(endpoint_full_url, params=_params,
                                             headers=_headers)
        else:
            json_data = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_ba5567f03dea5b6891957dd410319e3f_v2_2_2_3', json_data)

    def assign_device_to_site(self,
                              site_id,
                              device=None,
                              headers=None,
                              payload=None,
                              active_validation=True,
                              **request_parameters):
        """Assigns list of devices to a site .

        Args:
            device(list): Sites's device (list of objects).
            site_id(str): siteId path parameter. Site id to which site the device to assign .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(site_id, str,
                   may_be_none=False)
        if headers is not None:
            if '__runsync' in headers:
                check_type(headers.get('__runsync'),
                           bool, may_be_none=False)
            if '__persistbapioutput' in headers:
                check_type(headers.get('__persistbapioutput'),
                           bool, may_be_none=False)
            if '__runsynctimeout' in headers:
                check_type(headers.get('__runsynctimeout'),
                           int)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'siteId': site_id,
        }
        _payload = {
            'device':
                device,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_ada372b978e253228bdf7d3eab24b7a2_v2_2_2_3')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/system/api/v1/site/{siteId}/device')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload)

        return self._object_factory('bpm_ada372b978e253228bdf7d3eab24b7a2_v2_2_2_3', json_data)



# -*- coding: utf-8 -*-
"""Cisco DNA Center Tag API wrapper.

Copyright (c) 2019-2021 Cisco Systems.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from builtins import *


from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class Tag(object):
    """Cisco DNA Center Tag API (version: 2.3.5.3).

    Wraps the DNA Center Tag
    API and exposes the API as native Python
    methods that return native Python objects.

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new Tag
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the DNA Center service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(Tag, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def update_tag(self,
                   description=None,
                   dynamicRules=None,
                   id=None,
                   instanceTenantId=None,
                   name=None,
                   systemTag=None,
                   headers=None,
                   payload=None,
                   active_validation=True,
                   **request_parameters):
        """Updates a tag specified by id .

        Args:
            description(string): Tag's description.
            dynamicRules(list): Tag's dynamicRules (list of objects).
            id(string): Tag's id.
            instanceTenantId(string): Tag's instanceTenantId.
            name(string): Tag's name.
            systemTag(boolean): Tag's systemTag.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!update-tag
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        _payload = {
            'systemTag':
                systemTag,
            'description':
                description,
            'dynamicRules':
                dynamicRules,
            'name':
                name,
            'id':
                id,
            'instanceTenantId':
                instanceTenantId,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_c9f995abc21b54e7860f66aef2ffbc85_v2_3_5_3')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/tag')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload,
                                          headers=_headers)
        else:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload)

        return self._object_factory('bpm_c9f995abc21b54e7860f66aef2ffbc85_v2_3_5_3', json_data)

    def get_tag(self,
                additional_info_attributes=None,
                additional_info_name_space=None,
                field=None,
                level=None,
                limit=None,
                name=None,
                offset=None,
                order=None,
                size=None,
                sort_by=None,
                system_tag=None,
                headers=None,
                **request_parameters):
        """Returns the tags for given filter criteria .

        Args:
            name(str): name query parameter. Tag name is mandatory when filter operation is used. .
            additional_info_name_space(str): additionalInfo.nameSpace query parameter.
            additional_info_attributes(str): additionalInfo.attributes query parameter.
            level(str): level query parameter.
            offset(int,str): offset query parameter.
            limit(int,str): limit query parameter.
            size(str): size query parameter. size in kilobytes(KB) .
            field(str): field query parameter. Available field names are
                :'name,id,parentId,type,additionalInfo.nameSpace,additionalInfo.attributes' .
            sort_by(str): sortBy query parameter. Only supported attribute is name. SortyBy is mandatory when
                order is used. .
            order(str): order query parameter. Available values are asc and des .
            system_tag(str): systemTag query parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!get-tag
        """
        check_type(headers, dict)
        check_type(name, str)
        check_type(additional_info_name_space, str)
        check_type(additional_info_attributes, str)
        check_type(level, str)
        check_type(offset, (int, str))
        check_type(limit, (int, str))
        check_type(size, str)
        check_type(field, str)
        check_type(sort_by, str)
        check_type(order, str)
        check_type(system_tag, str)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'name':
                name,
            'additionalInfo.nameSpace':
                additional_info_name_space,
            'additionalInfo.attributes':
                additional_info_attributes,
            'level':
                level,
            'offset':
                offset,
            'limit':
                limit,
            'size':
                size,
            'field':
                field,
            'sortBy':
                sort_by,
            'order':
                order,
            'systemTag':
                system_tag,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/tag')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_a4185f5b40aabe991f8cdb2816_v2_3_5_3', json_data)

    def create_tag(self,
                   description=None,
                   dynamicRules=None,
                   id=None,
                   instanceTenantId=None,
                   name=None,
                   systemTag=None,
                   headers=None,
                   payload=None,
                   active_validation=True,
                   **request_parameters):
        """Creates tag with specified tag attributes .

        Args:
            description(string): Tag's description.
            dynamicRules(list): Tag's dynamicRules (list of objects).
            id(string): Tag's id.
            instanceTenantId(string): Tag's instanceTenantId.
            name(string): Tag's name.
            systemTag(boolean): Tag's systemTag.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!create-tag
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        _payload = {
            'systemTag':
                systemTag,
            'description':
                description,
            'dynamicRules':
                dynamicRules,
            'name':
                name,
            'id':
                id,
            'instanceTenantId':
                instanceTenantId,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_e8271b05b62c54609f74b4f2f373ad5a_v2_3_5_3')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/tag')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload)

        return self._object_factory('bpm_e8271b05b62c54609f74b4f2f373ad5a_v2_3_5_3', json_data)

    def get_tag_count(self,
                      attribute_name=None,
                      level=None,
                      name=None,
                      name_space=None,
                      size=None,
                      system_tag=None,
                      headers=None,
                      **request_parameters):
        """Returns tag count .

        Args:
            name(str): name query parameter.
            name_space(str): nameSpace query parameter.
            attribute_name(str): attributeName query parameter.
            level(str): level query parameter.
            size(str): size query parameter. size in kilobytes(KB) .
            system_tag(str): systemTag query parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!get-tag-count
        """
        check_type(headers, dict)
        check_type(name, str)
        check_type(name_space, str)
        check_type(attribute_name, str)
        check_type(level, str)
        check_type(size, str)
        check_type(system_tag, str)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'name':
                name,
            'nameSpace':
                name_space,
            'attributeName':
                attribute_name,
            'level':
                level,
            'size':
                size,
            'systemTag':
                system_tag,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/tag/count')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_afb52259f7c3501ca4d8ccd277828658_v2_3_5_3', json_data)

    def updates_tag_membership(self,
                               memberToTags=None,
                               memberType=None,
                               headers=None,
                               payload=None,
                               active_validation=True,
                               **request_parameters):
        """Updates tag membership. As part of the request payload through this API, only the specified members are added /
        retained to the given input tags. Possible values of memberType attribute in the request payload can be
        queried by using the /tag/member/type API .

        Args:
            memberToTags(object): Tag's memberToTags.
            memberType(string): Tag's memberType.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!updates-tag-membership
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        _payload = {
            'memberToTags':
                memberToTags,
            'memberType':
                memberType,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_e3934b0fb68a5ff787e65e9b7c8e6296_v2_3_5_3')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/tag/member')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload,
                                          headers=_headers)
        else:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload)

        return self._object_factory('bpm_e3934b0fb68a5ff787e65e9b7c8e6296_v2_3_5_3', json_data)

    def get_tag_resource_types(self,
                               headers=None,
                               **request_parameters):
        """Returns list of supported resource types .

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!get-tag-resource-types
        """
        check_type(headers, dict)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/tag/member/type')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_baf47897d525e5899f62e4d5bdd260b_v2_3_5_3', json_data)

    def delete_tag(self,
                   id,
                   headers=None,
                   **request_parameters):
        """Deletes a tag specified by id .

        Args:
            id(str): id path parameter. Tag ID .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!delete-tag
        """
        check_type(headers, dict)
        check_type(id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/tag/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.delete(endpoint_full_url, params=_params,
                                             headers=_headers)
        else:
            json_data = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_ed48fc373506cb1688cff36c2cb0f_v2_3_5_3', json_data)

    def get_tag_by_id(self,
                      id,
                      headers=None,
                      **request_parameters):
        """Returns tag specified by Id .

        Args:
            id(str): id path parameter. Tag ID .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!get-tag-by-id
        """
        check_type(headers, dict)
        check_type(id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/tag/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_d65f9b9d8ad5426bdf7e55461fcf761_v2_3_5_3', json_data)

    def get_tag_members_by_id(self,
                              id,
                              member_type,
                              level=None,
                              limit=None,
                              member_association_type=None,
                              offset=None,
                              headers=None,
                              **request_parameters):
        """Returns tag members specified by id .

        Args:
            id(str): id path parameter. Tag ID .
            member_type(str): memberType query parameter. Entity type of the member. Possible values can be
                retrieved by using /tag/member/type API .
            offset(str,int): offset query parameter. Used for pagination. It indicates the starting row number
                out of available member records .
            limit(str,int): limit query parameter. Used to Number of maximum members to return in the result .
            member_association_type(str): memberAssociationType query parameter. Indicates how the member is
                associated with the tag. Possible values and description. 1) DYNAMIC : The member is
                associated to the tag through rules. 2) STATIC – The member is associated to the tag
                manually. 3) MIXED – The member is associated manually and also satisfies the rule
                defined for the tag .
            level(str): level query parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!get-tag-members-by-id
        """
        check_type(headers, dict)
        check_type(member_type, str,
                   may_be_none=False)
        check_type(offset, (int, str))
        check_type(limit, (int, str))
        check_type(member_association_type, str)
        check_type(level, str)
        check_type(id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'memberType':
                member_type,
            'offset':
                offset,
            'limit':
                limit,
            'memberAssociationType':
                member_association_type,
            'level':
                level,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/tag/{id}/member')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_ff12c50ea3fb53c9a53f9c9e2c595d44_v2_3_5_3', json_data)

    def add_members_to_the_tag(self,
                               id,
                               object=None,
                               headers=None,
                               payload=None,
                               active_validation=True,
                               **request_parameters):
        """Adds members to the tag specified by id .

        Args:
            object(string): Tag's object.
            id(str): id path parameter. Tag ID .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!add-members-to-the-tag
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }
        _payload = {
            'object':
                object,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_dcc43be0514e50fea80cfa827f13ee5c_v2_3_5_3')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/tag/{id}/member')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload)

        return self._object_factory('bpm_dcc43be0514e50fea80cfa827f13ee5c_v2_3_5_3', json_data)

    def get_tag_member_count(self,
                             id,
                             member_type,
                             level=None,
                             member_association_type=None,
                             headers=None,
                             **request_parameters):
        """Returns the number of members in a given tag .

        Args:
            id(str): id path parameter. Tag ID .
            member_type(str): memberType query parameter.
            member_association_type(str): memberAssociationType query parameter.
            level(str): level query parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!get-tag-member-count
        """
        check_type(headers, dict)
        check_type(member_type, str,
                   may_be_none=False)
        check_type(member_association_type, str)
        check_type(level, str)
        check_type(id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'memberType':
                member_type,
            'memberAssociationType':
                member_association_type,
            'level':
                level,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/tag/{id}/member/count')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_ffacb52f745c15b40b9b352754e2e1_v2_3_5_3', json_data)

    def remove_tag_member(self,
                          id,
                          member_id,
                          headers=None,
                          **request_parameters):
        """Removes Tag member from the tag specified by id .

        Args:
            id(str): id path parameter. Tag ID .
            member_id(str): memberId path parameter. TagMember id to be removed from tag .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!remove-tag-member
        """
        check_type(headers, dict)
        check_type(id, str,
                   may_be_none=False)
        check_type(member_id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
            'memberId': member_id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/tag/{id}/member/{memberId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.delete(endpoint_full_url, params=_params,
                                             headers=_headers)
        else:
            json_data = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_cc9883be5c1cad1959347babb342_v2_3_5_3', json_data)



# -*- coding: utf-8 -*-
"""Cisco Catalyst Center Configuration Archive API wrapper.

Copyright (c) 2024 Cisco Systems.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""



from builtins import *



from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class ConfigurationArchive(object):
    """Cisco Catalyst Center Configuration Archive API (version: 2.3.7.9).

    Wraps the Catalyst Center Configuration Archive
    API and exposes the API as native Python
    methods that return native Python objects.

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new ConfigurationArchive
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Catalyst Center service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(ConfigurationArchive, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def export_device_configurations_v1(self,
                                        deviceId=None,
                                        password=None,
                                        headers=None,
                                        payload=None,
                                        active_validation=True,
                                        **request_parameters):
        """Export Device configuration for every device that is provided will be included in an encrypted zip file. .

        Args:
            deviceId(list): Configuration Archive's UUIDs of the devices for which configurations need to be
                exported.  (list of strings).
            password(string): Configuration Archive's Password for the zip file to protect exported configurations.
                Must contain, at minimum 8 characters, one lowercase letter, one uppercase letter, one
                number, one special character(-=[];,./~!@#$%^&*()_+{}|:?). It may not contain white
                space or the characters <>. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!export-device-configurations
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        _payload = {
            'deviceId':
                deviceId,
            'password':
                password,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_e85b40c5ca055f4c82281617a8f95644_v2_3_7_9')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/network-device-archive/cleartext')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload)

        return self._object_factory('bpm_e85b40c5ca055f4c82281617a8f95644_v2_3_7_9', json_data)

    def get_configuration_archive_details_v1(self,
                                             created_by=None,
                                             created_time=None,
                                             device_id=None,
                                             file_type=None,
                                             limit=None,
                                             offset=None,
                                             headers=None,
                                             **request_parameters):
        """Returns the historical device configurations (running configuration , startup configuration , vlan if
        applicable) by specified criteria .

        Args:
            device_id(str): deviceId query parameter. comma separated device id for example
                cf35b0a1-407f-412f-b2f4-f0c3156695f9,aaa38191-0c22-4158-befd-779a09d7cec1 . if device id
                is not provided it will fetch for all devices .
            file_type(str): fileType query parameter. Config File Type can be RUNNINGCONFIG or STARTUPCONFIG
                .
            created_time(str): createdTime query parameter. Supported with logical filters GT,GTE,LT,LTE & BT
                : time in milliseconds (epoc format) .
            created_by(str): createdBy query parameter. Comma separated values for createdBy SCHEDULED, USER,
                CONFIG_CHANGE_EVENT, SCHEDULED_FIRST_TIME, DR_CALL_BACK, PRE_DEPLOY .
            offset(int): offset query parameter.
            limit(int): limit query parameter. The number of records to be retrieved defaults to 500 if not
                specified, with a maximum allowed limit of 500. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            list: JSON response. A list of MyDict objects.
            Access the object's properties by using the dot notation
            or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!get-configuration-archive-details
        """
        check_type(headers, dict)
        check_type(device_id, str)
        check_type(file_type, str)
        check_type(created_time, str)
        check_type(created_by, str)
        check_type(offset, int)
        check_type(limit, int)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'deviceId':
                device_id,
            'fileType':
                file_type,
            'createdTime':
                created_time,
            'createdBy':
                created_by,
            'offset':
                offset,
            'limit':
                limit,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/network-device-config')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_ff699112d3854d99557dc1f48987f09_v2_3_7_9', json_data)

    def get_network_device_configuration_file_details_v1(self,
                                                         file_type=None,
                                                         id=None,
                                                         limit=None,
                                                         network_device_id=None,
                                                         offset=None,
                                                         headers=None,
                                                         **request_parameters):
        """Retrieves the list of network device configuration file details, sorted by createdTime in descending order. Use
        /intent/api/v1/networkDeviceConfigFiles/{id}/downloadMasked to download masked configurations, or
        /intent/api/v1/networkDeviceConfigFiles/{id}/downloadUnmasked for unmasked configurations. .

        Args:
            id(str): id query parameter. Unique identifier (UUID) of the configuration file. .
            network_device_id(str): networkDeviceId query parameter. Unique identifier (UUID) of the network
                devices. The number of networkDeviceId(s) must not exceed 5. .
            file_type(str): fileType query parameter. Type of device configuration file.Available values :
                'RUNNINGCONFIG', 'STARTUPCONFIG', 'VLAN' .
            offset(int): offset query parameter. The first record to show for this page; the first record is
                numbered 1. .
            limit(int): limit query parameter. The number of records to be retrieved defaults to 500 if not
                specified, with a maximum allowed limit of 500. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!get-network-device-configuration-file-details
        """
        check_type(headers, dict)
        check_type(id, str)
        check_type(network_device_id, str)
        check_type(file_type, str)
        check_type(offset, int)
        check_type(limit, int)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'id':
                id,
            'networkDeviceId':
                network_device_id,
            'fileType':
                file_type,
            'offset':
                offset,
            'limit':
                limit,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/networkDeviceConfigFiles')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_c07ca5c25f5084ae4148ce8b1ce940_v2_3_7_9', json_data)

    def count_of_network_device_configuration_files_v1(self,
                                                       file_type=None,
                                                       id=None,
                                                       network_device_id=None,
                                                       headers=None,
                                                       **request_parameters):
        """Retrieves count the details of the network device configuration files. .

        Args:
            id(str): id query parameter. Unique identifier (UUID) of the configuration file. .
            network_device_id(str): networkDeviceId query parameter. Unique identifier (UUID) of the network
                devices. The number of networkDeviceId(s) must not exceed 5. .
            file_type(str): fileType query parameter. Type of device configuration file. Available values :
                'RUNNINGCONFIG', 'STARTUPCONFIG', 'VLAN' .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!count-of-network-device-configuration-files
        """
        check_type(headers, dict)
        check_type(id, str)
        check_type(network_device_id, str)
        check_type(file_type, str)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'id':
                id,
            'networkDeviceId':
                network_device_id,
            'fileType':
                file_type,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/networkDeviceConfigFiles/count')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_af5e273c15f6abc150e9328e4d070_v2_3_7_9', json_data)

    def get_configuration_file_details_by_id_v1(self,
                                                id,
                                                headers=None,
                                                **request_parameters):
        """Retrieves the details of a specific network device configuration file using the `id`. .

        Args:
            id(str): id path parameter. The value of `id` can be obtained from the response of API
                `/dna/intent/api/v1/networkDeviceConfigFiles` .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!get-configuration-file-details-by-i-d
        """
        check_type(headers, dict)
        check_type(id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/networkDeviceConfigFiles/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_e8878000b5e5810be1b2630e70a5118_v2_3_7_9', json_data)

    def download_masked_device_configuration_v1(self,
                                                id,
                                                headers=None,
                                                **request_parameters):
        """Download the masked (sanitized) device configuration by providing the file `id`. .

        Args:
            id(str): id path parameter. The value of `id` can be obtained from the response of API
                `/dna/intent/api/v1/networkDeviceConfigFiles` .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!download-masked-device-configuration
        """
        check_type(headers, dict)
        check_type(id, str,
                   may_be_none=False)
        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/networkDeviceConfigFiles/{id}/downloa'
                 + 'dMasked')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params)

        return self._object_factory('bpm_fe0e28b3465084b5ee60a43602be1c_v2_3_7_9', json_data)

    def download_unmaskedraw_device_configuration_as_z_ip_v1(self,
                                                             id,
                                                             password=None,
                                                             headers=None,
                                                             payload=None,
                                                             active_validation=True,
                                                             **request_parameters):
        """Download the unmasked (raw) device configuration by providing the file `id` and a `password`. The response will
        be a password-protected zip file containing the unmasked configuration. Password must contain a minimum
        of 8 characters, one lowercase letter, one uppercase letter, one number, one special character
        (`-=[];,./~!@#$%^&*()_+{}|:?`). It may not contain white space or the characters `<>`. .

        Args:
            password(string): Configuration Archive's Password for the zip file to protect exported configurations.
                Must contain, at minimum 8 characters, one lowercase letter, one uppercase letter, one
                number, one special character(-=[];,./~!@#$%^&*()_+{}|:?). It may not contain white
                space or the characters <>. .
            id(str): id path parameter. The value of `id` can be obtained from the response of API
                `/dna/intent/api/v1/networkDeviceConfigFiles` .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!download-unmaskedraw-device-configuration-as-z-i-p
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(id, str,
                   may_be_none=False)
        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }
        _payload = {
            'password':
                password,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_d8fcd6dbb7ff53b58f7398c49b27ded2_v2_3_7_9')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/networkDeviceConfigFiles/{id}/downloa'
                 + 'dUnmasked')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload)

        return self._object_factory('bpm_d8fcd6dbb7ff53b58f7398c49b27ded2_v2_3_7_9', json_data)


    # Alias Function

    def download_unmaskedraw_device_configuration_as_z_ip(self,
                                                             id,
                                                             password=None,
                                                             headers=None,
                                                             payload=None,
                                                             active_validation=True,
                                                             **request_parameters):
        """ This function is an alias of download_unmaskedraw_device_configuration_as_z_ip_v1 .
        Args:
            password(string): Configuration Archive's Password for the zip file to protect exported configurations.
                Must contain, at minimum 8 characters, one lowercase letter, one uppercase letter, one
                number, one special character(-=[];,./~!@#$%^&*()_+{}|:?). It may not contain white
                space or the characters <>. .
            id(str): id path parameter. The value of `id` can be obtained from the response of API
                `/dna/intent/api/v1/networkDeviceConfigFiles` .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of download_unmaskedraw_device_configuration_as_z_ip_v1 .
        """
        return self.download_unmaskedraw_device_configuration_as_z_ip_v1(
                                                             id,
                                                             password,
                                                             headers,
                                                             payload,
                                                             active_validation,
                                                             **request_parameters)

    # Alias Function
    def export_device_configurations(self,
                                        deviceId=None,
                                        password=None,
                                        headers=None,
                                        payload=None,
                                        active_validation=True,
                                        **request_parameters):
        """ This function is an alias of export_device_configurations_v1 .
        Args:
            deviceId(list): Configuration Archive's UUIDs of the devices for which configurations need to be
                exported.  (list of strings).
            password(string): Configuration Archive's Password for the zip file to protect exported configurations.
                Must contain, at minimum 8 characters, one lowercase letter, one uppercase letter, one
                number, one special character(-=[];,./~!@#$%^&*()_+{}|:?). It may not contain white
                space or the characters <>. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of export_device_configurations_v1 .
        """
        return self.export_device_configurations_v1(
                    deviceId=deviceId,
                    password=password,
                    headers=headers,
                    payload=payload,
                    active_validation=active_validation,
                    **request_parameters
        )


    # Alias Function
    def get_network_device_configuration_file_details(self,
                                                         file_type=None,
                                                         id=None,
                                                         limit=None,
                                                         network_device_id=None,
                                                         offset=None,
                                                         headers=None,
                                                         **request_parameters):
        """ This function is an alias of get_network_device_configuration_file_details_v1 .
        Args:
            id(str): id query parameter. Unique identifier (UUID) of the configuration file. .
            network_device_id(str): networkDeviceId query parameter. Unique identifier (UUID) of the network
                devices. The number of networkDeviceId(s) must not exceed 5. .
            file_type(str): fileType query parameter. Type of device configuration file.Available values :
                'RUNNINGCONFIG', 'STARTUPCONFIG', 'VLAN' .
            offset(int): offset query parameter. The first record to show for this page; the first record is
                numbered 1. .
            limit(int): limit query parameter. The number of records to be retrieved defaults to 500 if not
                specified, with a maximum allowed limit of 500. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of get_network_device_configuration_file_details_v1 .
        """
        return self.get_network_device_configuration_file_details_v1(
                    file_type=file_type,
                    id=id,
                    limit=limit,
                    network_device_id=network_device_id,
                    offset=offset,
                    headers=headers,
                    **request_parameters
        )


    # Alias Function
    def count_of_network_device_configuration_files(self,
                                                       file_type=None,
                                                       id=None,
                                                       network_device_id=None,
                                                       headers=None,
                                                       **request_parameters):
        """ This function is an alias of count_of_network_device_configuration_files_v1 .
        Args:
            id(str): id query parameter. Unique identifier (UUID) of the configuration file. .
            network_device_id(str): networkDeviceId query parameter. Unique identifier (UUID) of the network
                devices. The number of networkDeviceId(s) must not exceed 5. .
            file_type(str): fileType query parameter. Type of device configuration file. Available values :
                'RUNNINGCONFIG', 'STARTUPCONFIG', 'VLAN' .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of count_of_network_device_configuration_files_v1 .
        """
        return self.count_of_network_device_configuration_files_v1(
                    file_type=file_type,
                    id=id,
                    network_device_id=network_device_id,
                    headers=headers,
                    **request_parameters
        )


    # Alias Function
    def get_configuration_archive_details(self,
                                             created_by=None,
                                             created_time=None,
                                             device_id=None,
                                             file_type=None,
                                             limit=None,
                                             offset=None,
                                             headers=None,
                                             **request_parameters):
        """ This function is an alias of get_configuration_archive_details_v1 .
        Args:
            device_id(str): deviceId query parameter. comma separated device id for example
                cf35b0a1-407f-412f-b2f4-f0c3156695f9,aaa38191-0c22-4158-befd-779a09d7cec1 . if device id
                is not provided it will fetch for all devices .
            file_type(str): fileType query parameter. Config File Type can be RUNNINGCONFIG or STARTUPCONFIG
                .
            created_time(str): createdTime query parameter. Supported with logical filters GT,GTE,LT,LTE & BT
                : time in milliseconds (epoc format) .
            created_by(str): createdBy query parameter. Comma separated values for createdBy SCHEDULED, USER,
                CONFIG_CHANGE_EVENT, SCHEDULED_FIRST_TIME, DR_CALL_BACK, PRE_DEPLOY .
            offset(int): offset query parameter.
            limit(int): limit query parameter. The number of records to be retrieved defaults to 500 if not
                specified, with a maximum allowed limit of 500. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of get_configuration_archive_details_v1 .
        """
        return self.get_configuration_archive_details_v1(
                    created_by=created_by,
                    created_time=created_time,
                    device_id=device_id,
                    file_type=file_type,
                    limit=limit,
                    offset=offset,
                    headers=headers,
                    **request_parameters
        )


    # Alias Function
    def get_configuration_file_details_by_id(self,
                                                id,
                                                headers=None,
                                                **request_parameters):
        """ This function is an alias of get_configuration_file_details_by_id_v1 .
        Args:
            id(str): id path parameter. The value of `id` can be obtained from the response of API
                `/dna/intent/api/v1/networkDeviceConfigFiles` .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of get_configuration_file_details_by_id_v1 .
        """
        return self.get_configuration_file_details_by_id_v1(
                    id=id,
                    headers=headers,
                    **request_parameters
        )


    # Alias Function
    def download_masked_device_configuration(self,
                                                id,
                                                headers=None,
                                                **request_parameters):
        """ This function is an alias of download_masked_device_configuration_v1 .
        Args:
            id(str): id path parameter. The value of `id` can be obtained from the response of API
                `/dna/intent/api/v1/networkDeviceConfigFiles` .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of download_masked_device_configuration_v1 .
        """
        return self.download_masked_device_configuration_v1(
                    id=id,
                    headers=headers,
                    **request_parameters
        )





# -*- coding: utf-8 -*-
"""Cisco Catalyst Center Applications API wrapper.

Copyright (c) 2024 Cisco Systems.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""



from builtins import *



from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class Applications(object):
    """Cisco Catalyst Center Applications API (version: 2.3.7.9).

    Wraps the Catalyst Center Applications
    API and exposes the API as native Python
    methods that return native Python objects.

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new Applications
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Catalyst Center service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(Applications, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def retrieves_the_list_of_network_applications_along_with_experience_and_health_metrics_v1(self,
                                                                                               site_id,
                                                                                               application_name=None,
                                                                                               attribute=None,
                                                                                               business_relevance=None,
                                                                                               end_time=None,
                                                                                               limit=None,
                                                                                               offset=None,
                                                                                               order=None,
                                                                                               sort_by=None,
                                                                                               ssid=None,
                                                                                               start_time=None,
                                                                                               headers=None,
                                                                                               **request_parameters):
        """Retrieves the list of network applications along with experience and health metrics. If startTime and endTime
        are not provided, the API defaults to the last 24 hours. `siteId` is mandatory. `siteId` must be a site
        UUID of a building. For detailed information about the usage of the API, please refer to the Open API
        specification document https://github.com/cisco-en-programmability/catalyst-center-api-
        specs/blob/main/Assurance/CE_Cat_Center_Org-NetworkApplications-1.0.0-resolved.yaml .

        Args:
            start_time(int): startTime query parameter. Start time from which API queries the data set related to
                the resource. It must be specified in UNIX epochtime in milliseconds. Value is
                inclusive. .
            end_time(int): endTime query parameter. End time to which API queries the data set related to the
                resource. It must be specified in UNIX epochtime in milliseconds. Value is inclusive. .
            limit(int): limit query parameter. Maximum number of records to return .
            offset(int): offset query parameter. Specifies the starting point within all records returned by the
                API. It's one based offset. The starting value is 1. .
            sort_by(str): sortBy query parameter. A field within the response to sort by. .
            order(str): order query parameter. The sort order of the field ascending or descending. .
            site_id(str): siteId query parameter. The site UUID without the top level hierarchy. `siteId` is
                mandatory. `siteId` must be a site UUID of a building. (Ex."buildingUuid") Examples:
                `siteId=buildingUuid` (single siteId requested)
                `siteId=buildingUuid1&siteId=buildingUuid2` (multiple siteId requested) .
            ssid(str): ssid query parameter. In the context of a network application, SSID refers to the name
                of the wireless network to which the client connects. Examples: `ssid=Alpha` (single
                ssid requested) `ssid=Alpha&ssid=Guest` (multiple ssid requested) .
            application_name(str): applicationName query parameter. Name of the application for which the
                experience data is intended. Examples: `applicationName=webex` (single applicationName
                requested) `applicationName=webex&applicationName=teams` (multiple applicationName
                requested) .
            business_relevance(str): businessRelevance query parameter. The application can be chosen to be
                categorized as business-relevant, irrelevant, or default (neutral). By doing so, the
                assurance application prioritizes the monitoring and analysis of business-relevant data,
                ensuring critical insights are captured. Applications marked as irrelevant or default
                are selectively excluded from certain data sets, streamlining focus on what's most
                important for business outcomes. .
            attribute(str): attribute query parameter. List of attributes related to resource that can be
                requested to only be part of the response along with the required attributes. Supported
                attributes are applicationName, siteId, exporterIpAddress, exporterNetworkDeviceId,
                healthScore, businessRelevance, usage, throughput, packetLossPercent, networkLatency,
                applicationServerLatency, clientNetworkLatency, serverNetworkLatency, trafficClass,
                jitter, ssid Examples: `attribute=healthScore` (single attribute requested)
                `attribute=healthScore&attribute=ssid&attribute=jitter` (multiple attribute requested) .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!retrieves-the-list-of-network-applications-along-with-experience-and-health-metrics
        """
        check_type(headers, dict)
        check_type(start_time, int)
        check_type(end_time, int)
        check_type(limit, int)
        check_type(offset, int)
        check_type(sort_by, str)
        check_type(order, str)
        check_type(site_id, str,
                   may_be_none=False)
        check_type(ssid, str)
        check_type(application_name, str)
        check_type(business_relevance, str)
        check_type(attribute, str)
        if headers is not None:
            if 'X-CALLER-ID' in headers:
                check_type(headers.get('X-CALLER-ID'),
                           str)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'startTime':
                start_time,
            'endTime':
                end_time,
            'limit':
                limit,
            'offset':
                offset,
            'sortBy':
                sort_by,
            'order':
                order,
            'siteId':
                site_id,
            'ssid':
                ssid,
            'applicationName':
                application_name,
            'businessRelevance':
                business_relevance,
            'attribute':
                attribute,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/data/api/v1/networkApplications')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_fb02436a6c935d5d8a536b86de8b2846_v2_3_7_9', json_data)

    def retrieves_the_total_count_of_network_applications_by_applying_basic_filtering_v1(self,
                                                                                         site_id,
                                                                                         application_name=None,
                                                                                         business_relevance=None,
                                                                                         end_time=None,
                                                                                         ssid=None,
                                                                                         start_time=None,
                                                                                         headers=None,
                                                                                         **request_parameters):
        """Retrieves the number of network applications by applying basic filtering. If startTime and endTime are not
        provided, the API defaults to the last 24 hours. `siteId` is mandatory. `siteId` must be a site UUID of
        a building. For detailed information about the usage of the API, please refer to the Open API
        specification document https://github.com/cisco-en-programmability/catalyst-center-api-
        specs/blob/main/Assurance/CE_Cat_Center_Org-NetworkApplications-1.0.0-resolved.yaml .

        Args:
            start_time(int): startTime query parameter. Start time from which API queries the data set related to
                the resource. It must be specified in UNIX epochtime in milliseconds. Value is
                inclusive. .
            end_time(int): endTime query parameter. End time to which API queries the data set related to the
                resource. It must be specified in UNIX epochtime in milliseconds. Value is inclusive. .
            site_id(str): siteId query parameter. The site UUID without the top level hierarchy. `siteId` is
                mandatory. `siteId` must be a site UUID of a building. (Ex."buildingUuid") Examples:
                `siteId=buildingUuid` (single siteId requested)
                `siteId=buildingUuid1&siteId=buildingUuid2` (multiple siteId requested) .
            ssid(str): ssid query parameter. In the context of a network application, SSID refers to the name
                of the wireless network to which the client connects. Examples: `ssid=Alpha` (single
                ssid requested) `ssid=Alpha&ssid=Guest` (multiple ssid requested) .
            application_name(str): applicationName query parameter. Name of the application for which the
                experience data is intended. Examples: `applicationName=webex` (single applicationName
                requested) `applicationName=webex&applicationName=teams` (multiple applicationName
                requested) .
            business_relevance(str): businessRelevance query parameter. The application can be chosen to be
                categorized as business-relevant, irrelevant, or default (neutral). By doing so, the
                assurance application prioritizes the monitoring and analysis of business-relevant data,
                ensuring critical insights are captured. Applications marked as irrelevant or default
                are selectively excluded from certain data sets, streamlining focus on what's most
                important for business outcomes. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!retrieves-the-total-count-of-network-applications-by-applying-basic-filtering
        """
        check_type(headers, dict)
        check_type(start_time, int)
        check_type(end_time, int)
        check_type(site_id, str,
                   may_be_none=False)
        check_type(ssid, str)
        check_type(application_name, str)
        check_type(business_relevance, str)
        if headers is not None:
            if 'X-CALLER-ID' in headers:
                check_type(headers.get('X-CALLER-ID'),
                           str)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'startTime':
                start_time,
            'endTime':
                end_time,
            'siteId':
                site_id,
            'ssid':
                ssid,
            'applicationName':
                application_name,
            'businessRelevance':
                business_relevance,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/data/api/v1/networkApplications/count')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_c50def6b3a58e5acab3ae592a57da8_v2_3_7_9', json_data)

    def retrieves_the_trend_analytics_data_related_to_network_applications_v1(self,
                                                                              aggregateAttributes=None,
                                                                              attributes=None,
                                                                              endTime=None,
                                                                              filters=None,
                                                                              groupBy=None,
                                                                              page=None,
                                                                              siteIds=None,
                                                                              startTime=None,
                                                                              trendInterval=None,
                                                                              headers=None,
                                                                              payload=None,
                                                                              active_validation=True,
                                                                              **request_parameters):
        """Retrieves the trend analytics of applications experience data for the specified time range. The data will be
        grouped based on the given trend time interval. This API facilitates obtaining consolidated insights
        into the performance and status of the network applications over the specified start and end time. If
        startTime and endTime are not provided, the API defaults to the last 24 hours. `siteId` and
        `trendInterval` are mandatory. `siteId` must be a site UUID of a building. For detailed information
        about the usage of the API, please refer to the Open API specification document
        https://github.com/cisco-en-programmability/catalyst-center-api-
        specs/blob/main/Assurance/CE_Cat_Center_Org-NetworkApplications-1.0.0-resolved.yaml .

        Args:
            aggregateAttributes(list): Applications's aggregateAttributes (list of objects).
            attributes(list): Applications's Attributes (list of strings).
            endTime(integer): Applications's End Time.
            filters(list): Applications's filters (list of objects).
            groupBy(list): Applications's Group By (list of strings).
            page(object): Applications's page.
            siteIds(list): Applications's Site Ids (list of strings).
            startTime(integer): Applications's Start Time.
            trendInterval(string): Applications's Trend Interval.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!retrieves-the-trend-analytics-data-related-to-network-applications
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'X-CALLER-ID' in headers:
                check_type(headers.get('X-CALLER-ID'),
                           str)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        _payload = {
            'startTime':
                startTime,
            'endTime':
                endTime,
            'siteIds':
                siteIds,
            'trendInterval':
                trendInterval,
            'groupBy':
                groupBy,
            'attributes':
                attributes,
            'filters':
                filters,
            'aggregateAttributes':
                aggregateAttributes,
            'page':
                page,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_6ce35f19bc4c1d058aa01536_v2_3_7_9')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/data/api/v1/networkApplications/trendAnalytics')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload)

        return self._object_factory('bpm_6ce35f19bc4c1d058aa01536_v2_3_7_9', json_data)

    def applications_v1(self,
                        application_health=None,
                        application_name=None,
                        device_id=None,
                        end_time=None,
                        limit=None,
                        mac_address=None,
                        offset=None,
                        site_id=None,
                        start_time=None,
                        headers=None,
                        **request_parameters):
        """Intent API to get a list of applications for a specific site, a device, or a client device's MAC address. For a
        combination of a specific application with site and/or device the API gets list of
        issues/devices/endpoints. .

        Args:
            site_id(str): siteId query parameter. Assurance site UUID value (Cannot be submitted together
                with deviceId and clientMac) .
            device_id(str): deviceId query parameter. Assurance device UUID value (Cannot be submitted
                together with siteId and clientMac) .
            mac_address(str): macAddress query parameter. Client device's MAC address (Cannot be submitted
                together with siteId and deviceId) .
            start_time(int): startTime query parameter. Starting epoch time in milliseconds of time window .
            end_time(int): endTime query parameter. Ending epoch time in milliseconds of time window .
            application_health(str): applicationHealth query parameter. Application health category (POOR,
                FAIR, or GOOD.  Optionally use with siteId only) .
            offset(int): offset query parameter. The offset of the first application in the returned data
                (optionally used with siteId only) .
            limit(int): limit query parameter. The max number of application entries in returned data [1, 1000]
                (optionally used with siteId only) .
            application_name(str): applicationName query parameter. The name of the application to get
                information on .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!applications
        """
        check_type(headers, dict)
        check_type(site_id, str)
        check_type(device_id, str)
        check_type(mac_address, str)
        check_type(start_time, int)
        check_type(end_time, int)
        check_type(application_health, str)
        check_type(offset, int)
        check_type(limit, int)
        check_type(application_name, str)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'siteId':
                site_id,
            'deviceId':
                device_id,
            'macAddress':
                mac_address,
            'startTime':
                start_time,
            'endTime':
                end_time,
            'applicationHealth':
                application_health,
            'offset':
                offset,
            'limit':
                limit,
            'applicationName':
                application_name,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/application-health')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_b85e4ce533d5ff49ddd3b2f9657cfa5_v2_3_7_9', json_data)



    # Alias Function
    def retrieves_the_trend_analytics_data_related_to_network_applications(self,
                                                                              aggregateAttributes=None,
                                                                              attributes=None,
                                                                              endTime=None,
                                                                              filters=None,
                                                                              groupBy=None,
                                                                              page=None,
                                                                              siteIds=None,
                                                                              startTime=None,
                                                                              trendInterval=None,
                                                                              headers=None,
                                                                              payload=None,
                                                                              active_validation=True,
                                                                              **request_parameters):
        """ This function is an alias of retrieves_the_trend_analytics_data_related_to_network_applications_v1 .
        Args:
            aggregateAttributes(list): Applications's aggregateAttributes (list of objects).
            attributes(list): Applications's Attributes (list of strings).
            endTime(integer): Applications's End Time.
            filters(list): Applications's filters (list of objects).
            groupBy(list): Applications's Group By (list of strings).
            page(object): Applications's page.
            siteIds(list): Applications's Site Ids (list of strings).
            startTime(integer): Applications's Start Time.
            trendInterval(string): Applications's Trend Interval.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of retrieves_the_trend_analytics_data_related_to_network_applications_v1 .
        """
        return self.retrieves_the_trend_analytics_data_related_to_network_applications_v1(
                    aggregateAttributes=aggregateAttributes,
                    attributes=attributes,
                    endTime=endTime,
                    filters=filters,
                    groupBy=groupBy,
                    page=page,
                    siteIds=siteIds,
                    startTime=startTime,
                    trendInterval=trendInterval,
                    headers=headers,
                    payload=payload,
                    active_validation=active_validation,
                    **request_parameters
        )


    # Alias Function
    def retrieves_the_list_of_network_applications_along_with_experience_and_health_metrics(self,
                                                                                               site_id,
                                                                                               application_name=None,
                                                                                               attribute=None,
                                                                                               business_relevance=None,
                                                                                               end_time=None,
                                                                                               limit=None,
                                                                                               offset=None,
                                                                                               order=None,
                                                                                               sort_by=None,
                                                                                               ssid=None,
                                                                                               start_time=None,
                                                                                               headers=None,
                                                                                               **request_parameters):
        """ This function is an alias of retrieves_the_list_of_network_applications_along_with_experience_and_health_metrics_v1 .
        Args:
            start_time(int): startTime query parameter. Start time from which API queries the data set related to
                the resource. It must be specified in UNIX epochtime in milliseconds. Value is
                inclusive. .
            end_time(int): endTime query parameter. End time to which API queries the data set related to the
                resource. It must be specified in UNIX epochtime in milliseconds. Value is inclusive. .
            limit(int): limit query parameter. Maximum number of records to return .
            offset(int): offset query parameter. Specifies the starting point within all records returned by the
                API. It's one based offset. The starting value is 1. .
            sort_by(str): sortBy query parameter. A field within the response to sort by. .
            order(str): order query parameter. The sort order of the field ascending or descending. .
            site_id(str): siteId query parameter. The site UUID without the top level hierarchy. `siteId` is
                mandatory. `siteId` must be a site UUID of a building. (Ex."buildingUuid") Examples:
                `siteId=buildingUuid` (single siteId requested)
                `siteId=buildingUuid1&siteId=buildingUuid2` (multiple siteId requested) .
            ssid(str): ssid query parameter. In the context of a network application, SSID refers to the name
                of the wireless network to which the client connects. Examples: `ssid=Alpha` (single
                ssid requested) `ssid=Alpha&ssid=Guest` (multiple ssid requested) .
            application_name(str): applicationName query parameter. Name of the application for which the
                experience data is intended. Examples: `applicationName=webex` (single applicationName
                requested) `applicationName=webex&applicationName=teams` (multiple applicationName
                requested) .
            business_relevance(str): businessRelevance query parameter. The application can be chosen to be
                categorized as business-relevant, irrelevant, or default (neutral). By doing so, the
                assurance application prioritizes the monitoring and analysis of business-relevant data,
                ensuring critical insights are captured. Applications marked as irrelevant or default
                are selectively excluded from certain data sets, streamlining focus on what's most
                important for business outcomes. .
            attribute(str): attribute query parameter. List of attributes related to resource that can be
                requested to only be part of the response along with the required attributes. Supported
                attributes are applicationName, siteId, exporterIpAddress, exporterNetworkDeviceId,
                healthScore, businessRelevance, usage, throughput, packetLossPercent, networkLatency,
                applicationServerLatency, clientNetworkLatency, serverNetworkLatency, trafficClass,
                jitter, ssid Examples: `attribute=healthScore` (single attribute requested)
                `attribute=healthScore&attribute=ssid&attribute=jitter` (multiple attribute requested) .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of retrieves_the_list_of_network_applications_along_with_experience_and_health_metrics_v1 .
        """
        return self.retrieves_the_list_of_network_applications_along_with_experience_and_health_metrics_v1(
                    site_id=site_id,
                    application_name=application_name,
                    attribute=attribute,
                    business_relevance=business_relevance,
                    end_time=end_time,
                    limit=limit,
                    offset=offset,
                    order=order,
                    sort_by=sort_by,
                    ssid=ssid,
                    start_time=start_time,
                    headers=headers,
                    **request_parameters
        )


    # Alias Function
    def retrieves_the_total_count_of_network_applications_by_applying_basic_filtering(self,
                                                                                         site_id,
                                                                                         application_name=None,
                                                                                         business_relevance=None,
                                                                                         end_time=None,
                                                                                         ssid=None,
                                                                                         start_time=None,
                                                                                         headers=None,
                                                                                         **request_parameters):
        """ This function is an alias of retrieves_the_total_count_of_network_applications_by_applying_basic_filtering_v1 .
        Args:
            start_time(int): startTime query parameter. Start time from which API queries the data set related to
                the resource. It must be specified in UNIX epochtime in milliseconds. Value is
                inclusive. .
            end_time(int): endTime query parameter. End time to which API queries the data set related to the
                resource. It must be specified in UNIX epochtime in milliseconds. Value is inclusive. .
            site_id(str): siteId query parameter. The site UUID without the top level hierarchy. `siteId` is
                mandatory. `siteId` must be a site UUID of a building. (Ex."buildingUuid") Examples:
                `siteId=buildingUuid` (single siteId requested)
                `siteId=buildingUuid1&siteId=buildingUuid2` (multiple siteId requested) .
            ssid(str): ssid query parameter. In the context of a network application, SSID refers to the name
                of the wireless network to which the client connects. Examples: `ssid=Alpha` (single
                ssid requested) `ssid=Alpha&ssid=Guest` (multiple ssid requested) .
            application_name(str): applicationName query parameter. Name of the application for which the
                experience data is intended. Examples: `applicationName=webex` (single applicationName
                requested) `applicationName=webex&applicationName=teams` (multiple applicationName
                requested) .
            business_relevance(str): businessRelevance query parameter. The application can be chosen to be
                categorized as business-relevant, irrelevant, or default (neutral). By doing so, the
                assurance application prioritizes the monitoring and analysis of business-relevant data,
                ensuring critical insights are captured. Applications marked as irrelevant or default
                are selectively excluded from certain data sets, streamlining focus on what's most
                important for business outcomes. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of retrieves_the_total_count_of_network_applications_by_applying_basic_filtering_v1 .
        """
        return self.retrieves_the_total_count_of_network_applications_by_applying_basic_filtering_v1(
                    site_id=site_id,
                    application_name=application_name,
                    business_relevance=business_relevance,
                    end_time=end_time,
                    ssid=ssid,
                    start_time=start_time,
                    headers=headers,
                    **request_parameters
        )


    # Alias Function
    def applications(self,
                        application_health=None,
                        application_name=None,
                        device_id=None,
                        end_time=None,
                        limit=None,
                        mac_address=None,
                        offset=None,
                        site_id=None,
                        start_time=None,
                        headers=None,
                        **request_parameters):
        """ This function is an alias of applications_v1 .
        Args:
            site_id(str): siteId query parameter. Assurance site UUID value (Cannot be submitted together
                with deviceId and clientMac) .
            device_id(str): deviceId query parameter. Assurance device UUID value (Cannot be submitted
                together with siteId and clientMac) .
            mac_address(str): macAddress query parameter. Client device's MAC address (Cannot be submitted
                together with siteId and deviceId) .
            start_time(int): startTime query parameter. Starting epoch time in milliseconds of time window .
            end_time(int): endTime query parameter. Ending epoch time in milliseconds of time window .
            application_health(str): applicationHealth query parameter. Application health category (POOR,
                FAIR, or GOOD.  Optionally use with siteId only) .
            offset(int): offset query parameter. The offset of the first application in the returned data
                (optionally used with siteId only) .
            limit(int): limit query parameter. The max number of application entries in returned data [1, 1000]
                (optionally used with siteId only) .
            application_name(str): applicationName query parameter. The name of the application to get
                information on .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of applications_v1 .
        """
        return self.applications_v1(
                    application_health=application_health,
                    application_name=application_name,
                    device_id=device_id,
                    end_time=end_time,
                    limit=limit,
                    mac_address=mac_address,
                    offset=offset,
                    site_id=site_id,
                    start_time=start_time,
                    headers=headers,
                    **request_parameters
        )


